# -*- coding: utf-8 -*-

# MIT License
# 
# Copyright (c) 2017 Sean-Jiun Wang
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
cms_figure
==========

This module provides classes and functions for styling plots made
using ROOT according to the CMS Publication Committee standards.

:copyright: (c) 2017 by Sean-Jiun Wang.
:license: MIT, see LICENSE for more details.
"""

__version__ = '0.9.0'

# Core classes
from .labels import CMSLabel, LuminosityLabel

from .tdr_style import TDRStyle

# Utilities
from .utils import draw_labels

__all__ = [
    # Core classes
    'CMSLabel', 'LuminosityLabel', 'TDRStyle',

    # Utilities
    'draw_labels',
]

